
const assert = require('assert')
const url = require('url')
const aws4 = require('aws4')
const axios = require('axios')
const debug = require('debug')('api-gateway')

class Client {
  constructor (config) {
    assert(config.accessKeyId)
    assert(config.secretAccessKey)
    assert(config.region)
    assert(config.endpoint)

    this.accessKeyId = config.accessKeyId
    this.secretAccessKey = config.secretAccessKey
    this.sessionToken = config.sessionToken
    this.region = config.region
    this.endpoint = config.endpoint
  }

  sign (request) {
    const url_object = url.parse(this.endpoint)
    const { headers = {}, method = 'GET', data } = request
    headers['X-Amz-Date'] = new Date().toISOString().replace(/\.\d{3}Z$/, 'Z').replace(/[:-]|\.\d{3}/g, '')
    const params = {
      service: 'execute-api',
      region: this.region,
      host: url_object.hostname,
      method: method.toUpperCase(),
      path: request.url,
      headers: headers
    }
    if (data) {
      params.body = JSON.stringify(data)
    }
    const credentials = {
      accessKeyId: this.accessKeyId,
      secretAccessKey: this.secretAccessKey,
      sessionToken: this.sessionToken
    }
    const signer = new aws4.RequestSigner(params, credentials)
    return signer.sign()
  }

  axios (request) {
    assert(request.method)
    assert(request.url)

    const { method, url, params, headers = {}, data } = request

    if (data && !headers['Content-Type']) {
      headers['Content-Type'] = 'application/json'
    }
    if (!headers['Accept']) {
      headers['Accept'] = 'application/json'
    }

    request.headers = headers
    const sign = this.sign(request)

    headers['X-Amz-Date'] = sign.headers['X-Amz-Date']
    headers['X-Amz-Security-Token'] = sign.headers['X-Amz-Security-Token']
    headers['Authorization'] = sign.headers['Authorization']

    const signed_request = {
      method: method,
      baseURL: this.endpoint,
      url: url,
      headers: headers,
      data: data,
      params: params
    }
    debug(signed_request)
    return axios(signed_request)
  }
}

module.exports = Client
