
require('dotenv').config()

const chai = require('chai')
const expect = chai.expect

const Client = require('..')

describe('api_client', function () {
  const params = {
    accessKeyId: process.env.AWS_ACCESS_KEY_ID,
    secretAccessKey: process.env.AWS_SECRET_ACCESS_KEY,
    sessionToken: process.env.AWS_SESSION_TOKEN,
    region: process.env.AWS_REGION || 'us-west-2',
    endpoint: process.env.ENDPOINT
  }
  const client = new Client(params)

  describe('sign', function () {
    it('shoud generate an API signature - GET', function () {
      const request = {
        url: process.env.URL
      }
      const signed = client.sign(request)
      expect(signed.headers['X-Amz-Security-Token']).to.equal(process.env.AWS_SESSION_TOKEN)
    })

    it('shoud generate an API signature - PUT', function () {
      const request = {
        method: 'put',
        url: process.env.URL,
        data: { cluster: 'abc' }
      }
      const signed = client.sign(request)
      expect(signed.headers['X-Amz-Security-Token']).to.equal(process.env.AWS_SESSION_TOKEN)
    })
  })

  describe('axios', function () {
    it('shoud make a signed get request to the API', function () {
      const request = {
        method: 'get',
        url: process.env.URL
      }
      return client.axios(request).then(res => {
        expect(res.data).to.be.an('object')
      })
    })

    it('shoud make a signed put request to the API', function () {
      const request = {
        method: 'put',
        url: process.env.URL,
        data: { cluster: 'abc' }
      }
      return client.axios(request).then(res => {
        expect(res.data).to.be.an('object')
      })
    })
  })
})
